/***************************************************
TASS Control Software by John Ackermann N8UR.  Based on Adafruit example below
****************************************************/

char version[] = "v0.32 -- 24 Aug. 2015";

// BOF preprocessor bug prevent - insert me on top of your arduino-code
// From: http://www.a-control.de/arduino-fehler/?lang=en
#if 1
__asm volatile ("nop");
#endif

// uncomment to enable Adafruit 2.8 inch TFT and capacitive touchscreen
#define USE_TS

// comment this to set board up as single 8 pole using K9
// #define SPLIT

#include "TASS_defines.h"

#ifdef USE_TS
  #include <Wire.h>
  #include <SPI.h>
  #include <Adafruit_GFX.h>
  #include <Adafruit_ILI9341.h>
  #include <Adafruit_FT6206.h>
#endif

// declare the tft and capacitive touchpad
#ifdef USE_TS
  Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
  Adafruit_FT6206 ctp = Adafruit_FT6206();
#endif

char board = ' ';
int relay = 0;
char cmd = ' ';
byte relay_num = 0;
boolean execute = 0;
boolean BadCmd = 0;

// these keep the relay settings for each board across commands
byte relay_mask_A = 0;
byte relay_mask_B = 0;
byte relay_mask_C = 0;
byte relay_mask_D = 0;

// these keep the hold settings for each board across commands
byte hold_mask_A = 0;
byte hold_mask_B = 0;
byte hold_mask_C = 0;
byte hold_mask_D = 0;

byte cmd_mask = 0;
byte relay_mask = 0;



// setup for serial input
const char startChar = '<';
const char endChar = '>';
char dataBuffer[DATABUFFERSIZE+1]; //Add 1 for NULL terminator

void setup(void) {
 
// set configuration pins to input pullup
  pinMode(14,INPUT_PULLUP);
  pinMode(15,INPUT_PULLUP);
  pinMode(16,INPUT_PULLUP);
  pinMode(17,INPUT_PULLUP);
  pinMode(18,INPUT_PULLUP);
  pinMode(19,INPUT_PULLUP);
  pinMode(22,INPUT_PULLUP);
              
  
// initialize serial:
  Serial.begin(115200);
// splash
  Serial.println("TAPR TASS Control Software");
  Serial.println(version);
  Serial.println("Copyright 2015 John Ackermann   N8UR");
  Serial.println("Licensed under MIT license");
  Serial.println("");

// set up digital pins for each board
  for (int relay_num=A_1;relay_num<=A_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=B_1;relay_num<=B_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=C_1;relay_num<=C_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  for (int relay_num=D_1;relay_num<=D_1+8;relay_num++){
    pinMode(relay_num, OUTPUT);}
  
#ifdef USE_TS
  tft_setup();
#endif

} // setup

void loop() {
  // command bytes
  char board = ' ';
  int relay = 0;
  char cmd = ' ';
  boolean BadCmd = 0;

  if ( execute ) {
   
  }
  
  if (!execute) {
    #ifdef USE_TS 
    get_ts_data();
    #endif
    get_usb_data();
 }
 if (execute) {
    board = toupper(dataBuffer[1]);
    relay = atoi(&dataBuffer[2]);
    cmd = toupper(dataBuffer[3]); 
    
    byte cmd_mask = 0;
    
  #ifdef SPLIT
    if (relay >=1 && relay <=4) {relay_mask = clear_mask_top(relay_mask);
    } else {relay_mask = clear_mask_bottom(relay_mask);}   
  #else
    relay_mask = clear_mask(relay_mask);
  #endif

    relay_mask = set_mask(relay, relay_mask);

    // if U clear appropriate hold mask
    if (cmd == 'U') {
      if (relay == 0) { relay_mask = 0; };
      switch (board) {
        case 'A': hold_mask_A = 0; break;
        case 'B': hold_mask_B = 0; break;
        case 'C': hold_mask_C = 0; break;
        case 'D': hold_mask_D = 0; break;
      }
   }

// if H set appropriate hold mask
    if (cmd == 'H') {
      switch (board) {
        case 'A':
          hold_mask_A = set_mask(relay,hold_mask_A);
          break;
        case 'B':
          hold_mask_B = set_mask(relay,hold_mask_B);
          break;
        case 'C':
          hold_mask_C = set_mask(relay,hold_mask_C);
          break;
        case 'D':
          hold_mask_D = set_mask(relay,hold_mask_D);
          break;
        } // switch
      } // if

// combine hold and relay to get command
    switch (board) {
    case 'A':
      write_board(A_1, hold_mask_A | relay_mask);
      break;
    case 'B':
      write_board(B_1, hold_mask_B | relay_mask);
      break;
    case 'C':
      write_board(C_1, hold_mask_C | relay_mask);
      break;
    case 'D':
      write_board(D_1, hold_mask_D | relay_mask);
      break;
    } // switch

  Serial.println("");
  Serial.print ("Board: ");
  Serial.print (board);
  Serial.print (" relay: ");
  Serial.print (relay);
  Serial.print (" command: ");
  Serial.print (cmd);
  if ((cmd == 'U') && (relay == 0)) { Serial.print(" -- Clear All"); }
  Serial.println("");
  
  #ifdef USE_TS
  draw_status();
  #endif
  execute = 0;
 } // execute 
} // of loop
  
byte set_mask(int x, byte y) {
  switch (x) {
    case 1: y |= 0x01; break;
    case 2: y |= 0x02; break;
    case 3: y |= 0x04; break;
    case 4: y |= 0x08; break;
    case 5: y |= 0x10; break;
    case 6: y |= 0x20; break;
    case 7: y |= 0x40; break;
    case 8: y |= 0x80; break;
  }
  return y;
} // of set_mask

byte clear_mask_top(byte mask) {
  mask = mask & 0xF0;
     return mask;   
}

byte clear_mask_bottom(byte mask) {
  mask = mask & 0x0F;
  return mask;  
}

byte clear_mask(byte mask) { 
  mask = mask = 0;
  return mask;  
}

void write_board(int first_io, byte cmd_mask) {
    int counter = first_io; // first relay
    for (byte mask = 00000001; mask>0; mask <<= 1) { //iterate through bit mask
      if (cmd_mask & mask) { // if bitwise AND resolves to true
        digitalWrite(counter,HIGH);
      } else {
        digitalWrite(counter,LOW);
      } // of if/else
      counter++;
    } // of for

  #ifndef SPLIT
  // set K9
  if (relay >4) {
    digitalWrite(first_io+8,HIGH);
  } else {
    digitalWrite(first_io+8,LOW);
  } 
  #endif
}  // of write_board

